import json


class PortalError(RuntimeError):
    """Base exception class
    """


class PortalConfigurationLoadError(PortalError):
    pass


class PortalAPIError(Exception):
    def __init__(self, code, message):
        self._code = code
        self._message = message

    @property
    def code(self):
        return self._code

    @property
    def message(self):
        return self._message

    @property
    def json(self):
        return json.dumps(self.message)

    def __str__(self):
        return self.__class__.__name__ + ': ' + self.message


class ResourceNotFoundError(PortalAPIError):
    def __init__(self, resource: str, res_id=None, note=None):
        message = dict(
            resource=resource,
            id=res_id,
            message=f"Cannot find {resource} with identifier: {res_id}",
        )
        if note:
            message['note'] = note

        super().__init__(code=404, message=message)


class UnauthorizedError(PortalAPIError):
    def __init__(self, note=None):
        message = dict(
            message=f"You are not authorized.",
        )
        if note:
            message['note'] = note

        super().__init__(code=401, message=message)


class ForbiddenError(PortalAPIError):
    # could use a resource identification (like 404)
    def __init__(self, uid=None, note=None):
        user_message = f"Forbidden for user: {uid}!" if uid else "Forbidden action."
        message = dict(
            uid=uid,
            message=user_message,
        )
        if note:
            message['note'] = note

        super().__init__(code=401, message=message)


class IncorrectPasswordError(UnauthorizedError):
    def __init__(self):
        super().__init__(note="Password is incorrect")


class ValidationError(PortalAPIError):  # currently not used
    def __init__(self, schema, data, failures: list):
        message = dict(
            schema=schema,
            data=data,
            failures=failures,
        )

        super().__init__(code=400, message=message)

